# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding field 'Post.broadcast'
        db.add_column('blog_post', 'broadcast', self.gf('django.db.models.fields.BooleanField')(default=True), keep_default=True)


    def backwards(self, orm):
        
        # Deleting field 'Post.broadcast'
        db.delete_column('blog_post', 'broadcast')


    models = {
        'attachments.attachment': {
            'Meta': {'object_name': 'Attachment'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mimetype': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'size': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'blog.category': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Category'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '100', 'db_index': 'True'})
        },
        'blog.post': {
            'Meta': {'ordering': "('-date',)", 'object_name': 'Post'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'blog_posts'", 'to': "orm['auth.User']"}),
            'body': ('django.db.models.fields.TextField', [], {}),
            'broadcast': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'categories': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'posts'", 'null': 'True', 'symmetrical': 'False', 'to': "orm['blog.Category']"}),
            'date': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '100', 'db_index': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'comments.comment': {
            'Meta': {'ordering': "('-sent',)", 'object_name': 'Comment'},
            'approved': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'body': ('django.db.models.fields.TextField', [], {}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'sent': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'spam': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'taggit.tag': {
            'Meta': {'object_name': 'Tag'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '100', 'db_index': 'True'})
        },
        'taggit.taggeditem': {
            'Meta': {'object_name': 'TaggedItem'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'taggit_taggeditem_tagged_items'", 'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.IntegerField', [], {'db_index': 'True'}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'taggit_taggeditem_items'", 'to': "orm['taggit.Tag']"})
        }
    }

    complete_apps = ['blog']



from django import forms
from django.conf import settings
from django.utils.timezone import now
from django.core.files import File
from django.template.defaultfilters import slugify
from bambu.blog.models import Post
from zipfile import ZipFile

class PostForm(forms.ModelForm):
	def __init__(self, *args, **kwargs):
		super(PostForm, self).__init__(*args, **kwargs)
		self.fields['date'].initial = now()
		self.fields['tags'].required = False
		
		if 'markitup' in settings.INSTALLED_APPS:
			from markitup.widgets import MarkItUpWidget
			self.fields['body'].widget = MarkItUpWidget()
		
		self.fields['css'].label = u'Custom CSS'
	
	class Meta:
		model = Post

class UploadForm(forms.ModelForm):
	zip = forms.FileField(label = u'Zip file', help_text = u'Upload your blog post')
	markdown = forms.BooleanField(
		label = u'Convert to Markdown',
		help_text = u'Convert post HTML to Markdown for ease-of-editing later',
		initial = True,
		required = False
	)
	
	def __init__(self, *args, **kwargs):
		super(UploadForm, self).__init__(*args, **kwargs)
		self.fields['tags'].required = False
	
	def clean_zip(self):
		try:
			self._zipfile = ZipFile(self.cleaned_data['zip'])
		except:
			raise forms.ValidationError('Upload a valid Zip file.')
		
		try:
			self._html = self._zipfile.read('post.html')
		except:
			raise forms.ValidationError('Zip file must contain at least one file, called post.html.')
	
	def save(self, commit = False):
		from pyquery import PyQuery as pq
		from dateutil import parser
		from tempfile import mkstemp
		from os import path, write, close, remove
		
		post = super(UploadForm, self).save(commit = False)
		post.body = 'Intermediate'
		post.date = now()
		post.published = False
		post.save()
		
		for category in self.cleaned_data.get('categories'):
			post.categories.add(category)
		
		for tag in self.cleaned_data.get('tags'):
			post.tags.add(tag)
		
		try:
			zipfile = self._zipfile
			html = pq(self._html)
			
			title = html('[data-bpfield="title"]').text()
			date = html('[data-bpfield="date"]').text()
			body = html('[data-bpfield="body"]')
			
			attachments = {}
			for img in body('[src]'):
				src = img.get('src')
				
				if not src:
					return
			
				if src.startswith('//') or src.startswith('http://') or src.startswith('https://'):
					continue
			
				try:
					zipped = zipfile.getinfo(src)
				except KeyError:
					continue
				
				handle, temp = mkstemp(path.splitext(src)[-1])
				write(handle, zipfile.open(src).read())
				close(handle)
				
				attachments[src] = 	post.attachments.create(
					file = File(open(temp, 'r')),
					title = src
				).file.url
				
				remove(temp)
				img = pq(img).attr('src', attachments[src])
				if not img.attr('alt'):
					img.attr('alt', src)
			
			post.title = title or '(No title)'
			
			try:
				post.date = parser.parse(date)
			except:
				pass
			
			if self.cleaned_data.get('markdown'):
				from html2text import HTML2Text
				writer = HTML2Text()
				writer.body_width = 0
				post.body = writer.handle(body.html())
			else:
				post.body = body.html().replace('\t', '')
			
			post.slug = slugify(post.title)
			
			css = html('[data-bpfield="css"]').text()
			if css:
				for old, new in attachments.items():
					css = css.replace(
						'(%s)' % old, "('%s')" % new
					).replace(
						"'%s'" % old, "'%s'" % new
					).replace(
						'"%s"' % old, '"%s"' % new
					)
				
				post.css = css
			
			post.save()
			return post
		except:
			post.delete()
			raise
	
	class Meta:
		model  = Post
		exclude = ('title', 'body', 'css', 'date', 'slug')